//-----------------------------------------------------------------------------
// Copyright (c) 2007 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//-----------------------------------------------------------------------------
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor: Xilinx
// \   \   \/    Version: 
//  \   \        Filename: v4_phy_controller_iobs.v
//  /   /        Date Last Modified: Oct 16 2007
// /___/   /\    Date Created: May 2 2005
// \   \  /  \
//  \___\/\___\
// 
//-----------------------------------------------------------------------------
// MPMC V4 MIG PHY Controller IOBs (Direct Clocking)
//-------------------------------------------------------------------------
//
// Description:
//   This module puts the memory control signals like address,
//   bank address, row address strobe, column address strobe,
//   write enable and clock enable in the IOBs.
//
// Structure:
//     
//--------------------------------------------------------------------------
//
// History:
//
//--------------------------------------------------------------------------

`timescale 1ns/1ps

module v4_phy_controller_iobs #
  (
   parameter integer row_address  = 14,
   parameter integer bank_address = 3,
   parameter integer cs_width     = 1,
   parameter integer cke_width    = 1,
   parameter integer odt_width    = 1  
   )
  (
   input                     CLK,
   input [row_address-1:0]   ctrl_ddr2_address,
   input [bank_address-1:0]  ctrl_ddr2_ba,
   input                     ctrl_ddr2_ras_L,
   input                     ctrl_ddr2_cas_L,
   input                     ctrl_ddr2_we_L,
   input [cs_width-1:0]      ctrl_ddr2_cs_L,
   input [cke_width-1:0]     ctrl_ddr2_cke,
   input [odt_width-1:0]     ctrl_ddr2_odt,

   output [row_address-1:0]  DDR_ADDRESS,
   output [bank_address-1:0] DDR_BA,
   output                    DDR_RAS_L,
   output                    DDR_CAS_L,
   output                    DDR_WE_L,
   output [cke_width-1:0]    DDR_CKE,
   output [odt_width-1:0]    DDR_ODT,
   output [cs_width-1:0]     ddr_cs_L,
   
   output [row_address-1:0]  DDR_ADDRESS_2,
   output [bank_address-1:0] DDR_BA_2,
   output                    DDR_RAS_L_2,
   output                    DDR_CAS_L_2,
   output                    DDR_WE_L_2,
   output [cke_width-1:0]    DDR_CKE_2,
   output [odt_width-1:0]    DDR_ODT_2,
   output [cs_width-1:0]     ddr_cs_L_2
   );
  
  (* IOB = "TRUE" *) reg [row_address-1:0]      ctrl_ddr2_address_d1;
  (* IOB = "TRUE" *) reg [bank_address-1:0]     ctrl_ddr2_ba_d1;
  (* IOB = "TRUE" *) reg                        ctrl_ddr2_ras_L_d1;
  (* IOB = "TRUE" *) reg                        ctrl_ddr2_cas_L_d1;
  (* IOB = "TRUE" *) reg                        ctrl_ddr2_we_L_d1;
  (* IOB = "TRUE" *) reg [cs_width-1:0]         ctrl_ddr2_cs_L_d1;
  (* IOB = "TRUE" *) reg [cke_width-1:0]        ctrl_ddr2_cke_d1;
  (* IOB = "TRUE" *) reg [odt_width-1:0]        ctrl_ddr2_odt_d1;

  (* IOB = "TRUE" *) reg [row_address-1:0]      ctrl_ddr2_address_d2;
  (* IOB = "TRUE" *) reg [bank_address-1:0]     ctrl_ddr2_ba_d2;
  (* IOB = "TRUE" *) reg                        ctrl_ddr2_ras_L_d2;
  (* IOB = "TRUE" *) reg                        ctrl_ddr2_cas_L_d2;
  (* IOB = "TRUE" *) reg                        ctrl_ddr2_we_L_d2;
  (* IOB = "TRUE" *) reg [cs_width-1:0]         ctrl_ddr2_cs_L_d2;
  (* IOB = "TRUE" *) reg [cke_width-1:0]        ctrl_ddr2_cke_d2;
  (* IOB = "TRUE" *) reg [odt_width-1:0]        ctrl_ddr2_odt_d2;
  
  wire [odt_width-1:0]       ctrl_ddr2_odt_tmp;
  genvar i;
  
  generate
    if (odt_width>cs_width) begin : gen_odt_rep
      for (i=0;i<odt_width/cs_width;i=i+1) begin : gen_odt
        assign ctrl_ddr2_odt_tmp[(i+1)*cs_width-1:i*cs_width] = 
                 ctrl_ddr2_odt[(i+1)*cs_width-1:i*cs_width] & ~ctrl_ddr2_cs_L;
      end
    end else begin : gen_odt_norep
      assign ctrl_ddr2_odt_tmp = ctrl_ddr2_odt & ~ctrl_ddr2_cs_L[odt_width-1:0];
    end
  endgenerate
    
  always @(posedge CLK)
    begin
      ctrl_ddr2_address_d1 <= ctrl_ddr2_address;
      ctrl_ddr2_ba_d1 <= ctrl_ddr2_ba;
      ctrl_ddr2_ras_L_d1 <= ctrl_ddr2_ras_L;
      ctrl_ddr2_cas_L_d1 <= ctrl_ddr2_cas_L;
      ctrl_ddr2_we_L_d1 <= ctrl_ddr2_we_L;
      ctrl_ddr2_cs_L_d1 <= ctrl_ddr2_cs_L;
      ctrl_ddr2_cke_d1 <= ctrl_ddr2_cke;
      ctrl_ddr2_odt_d1 <= ctrl_ddr2_odt_tmp;
      
      ctrl_ddr2_address_d2 <= ctrl_ddr2_address;
      ctrl_ddr2_ba_d2 <= ctrl_ddr2_ba;
      ctrl_ddr2_ras_L_d2 <= ctrl_ddr2_ras_L;
      ctrl_ddr2_cas_L_d2 <= ctrl_ddr2_cas_L;
      ctrl_ddr2_we_L_d2 <= ctrl_ddr2_we_L;
      ctrl_ddr2_cs_L_d2 <= ctrl_ddr2_cs_L;
      ctrl_ddr2_cke_d2 <= ctrl_ddr2_cke;
      ctrl_ddr2_odt_d2 <= ctrl_ddr2_odt_tmp;
    end
  
  OBUF r0
    (
     .I  (ctrl_ddr2_ras_L_d1),
     .O  (DDR_RAS_L)
     );
     
  OBUF r0_2
    (
     .I  (ctrl_ddr2_ras_L_d2),
     .O  (DDR_RAS_L_2)
     );
     
  OBUF r1
    (
     .I  (ctrl_ddr2_cas_L_d1),
     .O  (DDR_CAS_L)
     );
 
   OBUF r1_2
    (
     .I  (ctrl_ddr2_cas_L_d2),
     .O  (DDR_CAS_L_2)
     );
     
  OBUF r2
    (
     .I  (ctrl_ddr2_we_L_d1),
     .O  (DDR_WE_L)
     );
  
  OBUF r2_2
    (
     .I  (ctrl_ddr2_we_L_d2),
     .O  (DDR_WE_L_2)
     );
     
  generate
    for (i=0;i<cs_width;i=i+1)
      begin : gen_obuf_cs
        OBUF OBUF_cs
          (
           .I  (ctrl_ddr2_cs_L_d1[i]),
           .O  (ddr_cs_L[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<cs_width;i=i+1)
      begin : gen_obuf_cs_2
        OBUF OBUF_cs_2
          (
           .I  (ctrl_ddr2_cs_L_d2[i]),
           .O  (ddr_cs_L_2[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<cke_width;i=i+1)
      begin : gen_obuf_cke
        OBUF OBUF_cke
          (
           .I  (ctrl_ddr2_cke_d1[i]),
           .O  (DDR_CKE[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<cke_width;i=i+1)
      begin : gen_obuf_cke_2
        OBUF OBUF_cke_2
          (
           .I  (ctrl_ddr2_cke_d2[i]),
           .O  (DDR_CKE_2[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<odt_width;i=i+1)
      begin : gen_obuf_odt
        OBUF OBUF_odt
          (
           .I  (ctrl_ddr2_odt_d1[i]),
           .O  (DDR_ODT[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<odt_width;i=i+1)
      begin : gen_obuf_odt_2
        OBUF OBUF_odt_2
          (
           .I  (ctrl_ddr2_odt_d2[i]),
           .O  (DDR_ODT_2[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<row_address;i=i+1)
      begin : gen_obuf_r
        OBUF OBUF_r
          (
           .I  (ctrl_ddr2_address_d1[i]),
           .O  (DDR_ADDRESS[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<row_address;i=i+1)
      begin : gen_obuf_r_2
        OBUF OBUF_r_2
          (
           .I  (ctrl_ddr2_address_d2[i]),
           .O  (DDR_ADDRESS_2[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<bank_address;i=i+1)
      begin : gen_obuf_b
        OBUF OBUF_b
          (
           .I  (ctrl_ddr2_ba_d1[i]),
           .O  (DDR_BA[i])
           );
      end
  endgenerate
  
  generate
    for (i=0;i<bank_address;i=i+1)
      begin : gen_obuf_b_2
        OBUF OBUF_b_2
          (
           .I  (ctrl_ddr2_ba_d2[i]),
           .O  (DDR_BA_2[i])
           );
      end
  endgenerate
  
endmodule
